import { PresenceType, StatusCode } from '@ucap/core';

import { StatusBulkInfo } from '@ucap/protocol-status';

export class PresenceUtil {
  /**
   * Returns css class name of presence
   * Format
   *    ucap-organization-presence-none
   *    ucap-organization-presence-pc-online
   *    ucap-organization-presence-pc-offline
   *    ucap-organization-presence-pc-busy
   *    ucap-organization-presence-pc-away
   *    ucap-organization-presence-pc-away2
   *    ucap-organization-presence-pc-none
   *
   *    ucap-organization-presence-mobile-online
   *    ucap-organization-presence-mobile-offline
   *    ucap-organization-presence-mobile-busy
   *    ucap-organization-presence-mobile-away
   *    ucap-organization-presence-mobile-away2
   *    ucap-organization-presence-mobile-none
   *
   * @param statusBulkInfo status info.
   * @param presenceType presence type.
   */
  static cssClass(
    statusBulkInfo: StatusBulkInfo,
    presenceType: PresenceType
  ): string {
    const statusCode = PresenceUtil.getStatusCode(statusBulkInfo, presenceType);
    if (!statusCode) {
      return 'ucap-organization-presence-none';
    }

    let cssClass = `ucap-organization-presence-${presenceType}`;

    switch (statusCode) {
      case StatusCode.OnLine:
        cssClass = `${cssClass}-online`;
        break;
      case StatusCode.Offline:
        cssClass = `${cssClass}-offline`;
        break;
      case StatusCode.Busy:
        cssClass = `${cssClass}-busy`;
        break;
      case StatusCode.Away:
        cssClass = `${cssClass}-away`;
        break;
      case StatusCode.Away2:
        cssClass = `${cssClass}-away2`;
        break;
      default:
        cssClass = `${cssClass}-none`;
        break;
    }

    return cssClass;
  }

  /**
   * Returns whether online
   * @param statusBulkInfo status info.
   * @param presenceType presence type.
   */
  static isOnline(
    statusBulkInfo: StatusBulkInfo,
    presenceType: PresenceType
  ): boolean {
    const statusCode = PresenceUtil.getStatusCode(statusBulkInfo, presenceType);
    if (!statusCode) {
      return false;
    }

    return StatusCode.OnLine === statusCode;
  }

  /**
   * Returns whether offline
   * @param statusBulkInfo status info.
   * @param presenceType presence type.
   */
  static isOffline(
    statusBulkInfo: StatusBulkInfo,
    presenceType: PresenceType
  ): boolean {
    const statusCode = PresenceUtil.getStatusCode(statusBulkInfo, presenceType);
    if (!statusCode) {
      return false;
    }

    return StatusCode.Offline === statusCode;
  }

  /**
   * Returns whether busy
   * @param statusBulkInfo status info.
   * @param presenceType presence type.
   */
  static isBusy(
    statusBulkInfo: StatusBulkInfo,
    presenceType: PresenceType
  ): boolean {
    const statusCode = PresenceUtil.getStatusCode(statusBulkInfo, presenceType);
    if (!statusCode) {
      return false;
    }

    return StatusCode.Busy === statusCode;
  }

  /**
   * Returns whether away
   * @param statusBulkInfo status info.
   * @param presenceType presence type.
   */
  static isAway(
    statusBulkInfo: StatusBulkInfo,
    presenceType: PresenceType
  ): boolean {
    const statusCode = PresenceUtil.getStatusCode(statusBulkInfo, presenceType);
    if (!statusCode) {
      return false;
    }

    return StatusCode.Away === statusCode;
  }

  /**
   * Returns whether away2
   * @param statusBulkInfo status info.
   * @param presenceType presence type.
   */
  static isAway2(
    statusBulkInfo: StatusBulkInfo,
    presenceType: PresenceType
  ): boolean {
    const statusCode = PresenceUtil.getStatusCode(statusBulkInfo, presenceType);
    if (!statusCode) {
      return false;
    }

    return StatusCode.Away2 === statusCode;
  }

  private static getStatusCode(
    statusBulkInfo: StatusBulkInfo,
    presenceType: PresenceType
  ): StatusCode {
    if (!statusBulkInfo || !presenceType) {
      return undefined;
    }

    switch (presenceType) {
      case PresenceType.PC:
        return statusBulkInfo.pcStatus;
      case PresenceType.MOBILE:
        return statusBulkInfo.mobileStatus;
      case PresenceType.MOBILE_CONFERENCE:
        return statusBulkInfo.mobileConferenceStatus;
      case PresenceType.PHONE:
        return statusBulkInfo.phoneStatus;
      case PresenceType.CONFERENCE:
        return statusBulkInfo.conferenceStatus;
      case PresenceType.IMESSENGER:
        return statusBulkInfo.imessengerStatus;
      default:
        return undefined;
    }
  }
}
